<?xml version="1.0"?>

<!--

    This stylesheet transforms an XMI version 1.2 document into an html document that describes the classes, attributes, and associations
    in the underlying domain model.
    
    To use (assuming ArgoUML):
    
    1.  In ArgoUML, export the model as XMI (Tools menu, select "Export as XMI...")
    2.  Transform the output XMI file using an XSLT engine.
          For example, assuming Xalan is invoked with the command "xalan" and output XMI is "domain-model.xmi":
          xalan -in domain-model.xmi -xsl create-documentation-xsl -out documentation.html -html

-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xsd="http://www.w3.org/2001/XMLSchema" version="1.0">

    <xsl:template match="/XMI/XMI.content">
        <!-- start the html with the content section -->
        <xsl:variable name="model-name"><xsl:value-of select="Model_Management.Model/Foundation.Core.ModelElement.name"/></xsl:variable>
        <html style="font-family: helvetica, sans-serif">
            <head><title>Documentation for <xsl:value-of select="$model-name"/></title></head>
            <body>
                <h1>Documentation for Model: <xsl:value-of select="$model-name"/></h1>
                <xsl:apply-templates select="Model_Management.Model/Foundation.Core.Namespace.ownedElement"/>
            </body>
        </html>
    </xsl:template>
    
    <xsl:template match="Foundation.Core.Namespace.ownedElement">
        <!-- the main driver loop...process each ownedElement, which will basically wind up being a row in the output html table -->
        <p>In the table, class names are in the leftmost column, in <span style="font-weight: bold; color: darkblue">bold blue font</span>.  The class definition appears in the first row associated with the class,
            in <span style="color: darkblue">blue font</span>.  Associations are indicated in <i>italics</i>.  Missing definitions are indicated in <span style="color: darkred">red font</span>.</p>
        <table border="1" cellpadding="5">
            <tr style="background: #C0C0C0"><th>Class</th><th>Property or <i>Reference</i></th><th>Documentation</th><th>Cardinality</th></tr>
            <xsl:apply-templates select="Foundation.Core.Class|Foundation.Core.DataType|Foundation.Core.Association">
                <xsl:sort select="Foundation.Core.ModelElement.name"/>
            </xsl:apply-templates>
        </table>
    </xsl:template>
    
    <xsl:template match="Foundation.Core.Class">
    
        <!-- Process each class...handle the name of the class, and then each "attribute" (class property) and association -->
        
        <!-- First, define some XPaths that will be used more than once -->
        
        <xsl:variable name="id">
            <xsl:value-of select="@xmi.id"/>
        </xsl:variable>
        <xsl:variable name="attribute-count">
            <xsl:value-of select="count(Foundation.Core.Classifier.feature/Foundation.Core.Attribute)"/>
        </xsl:variable>
        <xsl:variable name="reference-count">
            <xsl:value-of select="count(//Foundation.Core.Association/Foundation.Core.Association.connection/Foundation.Core.AssociationEnd[Foundation.Core.AssociationEnd.isNavigable/@xmi.value='false' and Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref=$id])"/>
        </xsl:variable>
        <xsl:variable name="total-child-count">
            <xsl:value-of select="$attribute-count + $reference-count"/>
        </xsl:variable>
        <xsl:variable name="class-name">
            <xsl:value-of select="Foundation.Core.ModelElement.name"/>
        </xsl:variable>
        <xsl:variable name="class-docn" select="Foundation.Core.ModelElement.taggedValue/Foundation.Extension_Mechanisms.TaggedValue[Foundation.Extension_Mechanisms.TaggedValue.tag='documentation']/Foundation.Extension_Mechanisms.TaggedValue.value"/>
        
        <!-- handle special case of a class that has no attributes -->
        
        <xsl:if test="$total-child-count=0 and string-length($class-name) != 0">
            <xsl:call-template name="class-row">
                <xsl:with-param name="total-child-count" select="$total-child-count"/>
                <xsl:with-param name="class-name" select="$class-name"/>
                <xsl:with-param name="class-docn" select="$class-docn"/>
            </xsl:call-template>
        </xsl:if>
        
        <!-- handle each attribute -->
        
        <xsl:for-each select="Foundation.Core.Classifier.feature/Foundation.Core.Attribute">
            <!-- if it's the first attribute, write out the class header row -->
            <xsl:if test="position()=1">
                <xsl:call-template name="class-row">
                    <xsl:with-param name="total-child-count" select="$total-child-count"/>
                    <xsl:with-param name="class-name" select="$class-name"/>
                    <xsl:with-param name="class-docn" select="$class-docn"/>
                </xsl:call-template>
            </xsl:if>
            <tr>
                <!-- write the attribute row (the name of the attribute, documentation of the attribute, and cardinality) -->
                <td><xsl:value-of select="Foundation.Core.ModelElement.name"/></td>
                <xsl:call-template name="documentation-cell">
                    <xsl:with-param name="docn" select="Foundation.Core.ModelElement.taggedValue/Foundation.Extension_Mechanisms.TaggedValue[Foundation.Extension_Mechanisms.TaggedValue.tag='documentation']/Foundation.Extension_Mechanisms.TaggedValue.value"/>
                </xsl:call-template>
                <xsl:variable name="multiplicityRef"><xsl:value-of select="Foundation.Core.StructuralFeature.multiplicity/Foundation.Data_Types.Multiplicity/@xmi.idref"/></xsl:variable>
                <xsl:call-template name="cardinality-cell">
                    <xsl:with-param name="multiplicity" select="/XMI/XMI.content/Model_Management.Model//Foundation.Data_Types.Multiplicity[@xmi.id=$multiplicityRef]"/>
                </xsl:call-template>
            </tr>
        </xsl:for-each>
        
        <!-- handle each association -->
        
        <xsl:for-each select="//Foundation.Core.Association/Foundation.Core.Association.connection/Foundation.Core.AssociationEnd[Foundation.Core.AssociationEnd.isNavigable/@xmi.value='false' and Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier/@xmi.idref=$id]">
            
            <xsl:variable name="referenceToAssociatedClass"><xsl:value-of select="../../Foundation.Core.Association.connection/Foundation.Core.AssociationEnd[Foundation.Core.AssociationEnd.isNavigable/@xmi.value='true']/Foundation.Core.AssociationEnd.type/Foundation.Core.Classifier"/></xsl:variable>
            <xsl:variable name="associatedClass"><xsl:value-of select="//Foundation.Core.Class[@xmi.id=$referenceToAssociatedClass/@xmi.idref]"/></xsl:variable>
            
            <!-- handle special case of a class with no attributes (only associations), where this is the first association -->
            
            <xsl:if test="position()=1 and $attribute-count=0">
                <xsl:call-template name="class-row">
                    <xsl:with-param name="total-child-count" select="$total-child-count"/>
                    <xsl:with-param name="class-name" select="$class-name"/>
                    <xsl:with-param name="class-docn" select="$class-docn"/>
                </xsl:call-template>
            </xsl:if>
            <tr>
                <!-- write the association row (name and link to associated class, documentation, and cardinality) -->
                <td><i>
                    <a>
                        <xsl:attribute name="href"><xsl:text>#</xsl:text><xsl:value-of select="$associatedClass/Foundation.Core.ModelElement.name"/></xsl:attribute>
                        <xsl:value-of select="$associatedClass/Foundation.Core.ModelElement.name"/>
                        <xsl:if test="../../Foundation.Core.ModelElement.name and string-length(../../Foundation.Core.ModelElement.name) != 0">
                            <xsl:text> (</xsl:text><xsl:value-of select="../../Foundation.Core.ModelElement.name"/><xsl:text>)</xsl:text>
                        </xsl:if>
                    </a>
                </i></td>
                <xsl:call-template name="documentation-cell">
                    <xsl:with-param name="docn" select="../../Foundation.Core.ModelElement.taggedValue/Foundation.Extension_Mechanisms.TaggedValue[Foundation.Extension_Mechanisms.TaggedValue.tag='documentation']/Foundation.Extension_Mechanisms.TaggedValue.value"/>
                </xsl:call-template>
                <xsl:variable name="multiplicityRef"><xsl:value-of select="../../Foundation.Core.Association.connection/Foundation.Core.AssociationEnd[Foundation.Core.AssociationEnd.isNavigable/@xmi.value='true']/Foundation.Core.AssociationEnd.multiplicity/Foundation.Data_Types.Multiplicity/@xmi.idref"/></xsl:variable>
                <xsl:call-template name="cardinality-cell">
                    <xsl:with-param name="multiplicity" select="/XMI/XMI.content/Model_Management.Model//Foundation.Data_Types.Multiplicity[@xmi.id=$multiplicityRef]"/>
                </xsl:call-template>
            </tr>
        </xsl:for-each>
        
    </xsl:template>
    
    <!-- template called from above to handle formatting of documentation cells -->
    
    <xsl:template name="documentation-cell">
        <xsl:param name="docn"/>
        <td>
            <xsl:choose>
                <xsl:when test="$docn and string-length($docn) != 0">
                    <xsl:value-of select="$docn"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="missing-defn-text"/>
                </xsl:otherwise>
            </xsl:choose>
        </td>
    </xsl:template>
    
    <!-- template called from above to handle formatting of cardinality cells -->
    
    <xsl:template name="cardinality-cell">
        <xsl:param name="multiplicity"/>
        <xsl:variable name="minOccurs"><xsl:value-of select="$multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.lower"/></xsl:variable>
        <xsl:variable name="maxOccurs"><xsl:value-of select="$multiplicity/Foundation.Data_Types.Multiplicity.range/Foundation.Data_Types.MultiplicityRange/Foundation.Data_Types.MultiplicityRange.upper"/></xsl:variable>
        <td>
            <xsl:value-of select="$minOccurs"/>
            <xsl:text>,</xsl:text>
            <xsl:choose>
                <xsl:when test="$maxOccurs='-1'"><xsl:text>*</xsl:text></xsl:when>
                <xsl:otherwise><xsl:value-of select="$maxOccurs"/></xsl:otherwise>
            </xsl:choose>
        </td>
    </xsl:template>
    
    <!-- template called from above to handle formatting of class header rows -->
    
    <xsl:template name="class-row">
        <xsl:param name="total-child-count"/>
        <xsl:param name="class-name"/>
        <xsl:param name="class-docn"/>
        <tr>
            <td style="font-weight: bold; color: darkblue">
                <xsl:attribute name="rowspan"><xsl:value-of select="$total-child-count + 1"/></xsl:attribute>
                <a>
                    <xsl:attribute name="name"><xsl:value-of select="$class-name"/></xsl:attribute>
                    <xsl:value-of select="$class-name"/>
                </a>
            </td>
            <td colspan="3">
                <xsl:choose>
                    <xsl:when test="$class-docn and string-length($class-docn) != 0">
                        <span style="color: darkblue"><xsl:value-of select="$class-docn"/></span>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:call-template name="missing-defn-text"/>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
        </tr>
    </xsl:template>
    
    <!-- template called from above to handle text representing a missing definition -->
    
    <xsl:template name="missing-defn-text">
        <span style="color: darkred">Definition needed.</span>
    </xsl:template>

    <!-- some templates that identify structures that we don't want to process -->
    
    <xsl:template match="/XMI/XMI.header"/>
    <xsl:template match="Foundation.Core.DataType"/>
    <xsl:template match="Foundation.Core.Association"/>
    
</xsl:stylesheet>

<!--

Copyright (C) 2005 Justice Integration Solutions, Inc. All rights reserved.

Redistribution and use in source and binary forms, with or without modifica-
tion, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice,
   this list of conditions, and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
(INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

-->
